@@ -35,6 +35,7 @@
 #include <shlobj.h>
 #include <time.h>
 #include <sddl.h>
+#include <stdio.h>  // MSVC requires this for _vsnprintf
 // #include <KnownFolders.h>
 #else
 #include <pwd.h>
@@ -50,8 +51,10 @@
 #include <algorithm>
 #include <cctype>
 #include <cerrno>
+#include <cstdarg>
 #include <fstream>
 #include <iterator>
+#include <limits>
 #include <map>
 #include <sstream>
 #include <string>
@@ -70,6 +73,7 @@
 #include "base/mac_util.h"
 #endif
 
+
 namespace {
 
 const char kFileDelimiterForUnix = '/';
@@ -77,6 +81,8 @@ const char kFileDelimiterForWindows = '\\';
 
 #ifdef OS_WINDOWS
 const char kFileDelimiter = kFileDelimiterForWindows;
+volatile mozc::Util::IsWindowsX64Mode g_is_windows_x64_mode =
+    mozc::Util::IS_WINDOWS_X64_DEFAULT_MODE;
 #else
 const char kFileDelimiter = kFileDelimiterForUnix;
 #endif
@@ -127,7 +133,6 @@ void SplitStringToIteratorUsing(const string &full,
   }
 }
 
-
 template <typename ITR>
 void SplitStringToIteratorAllowEmpty(const string &full,
                                      const char *delim,
@@ -146,6 +151,54 @@ void SplitStringToIteratorAllowEmpty(const string &full,
   }
 }
 
+// Lower-level routine that takes a va_list and appends to a specified
+// string.  All other routines of sprintf family are just convenience
+// wrappers around it.
+void StringAppendV(string* dst, const char* format, va_list ap) {
+  // First try with a small fixed size buffer
+  char space[1024];
+
+  // It's possible for methods that use a va_list to invalidate
+  // the data in it upon use.  The fix is to make a copy
+  // of the structure before using it and use that copy instead.
+  va_list backup_ap;
+  va_copy(backup_ap, ap);
+  int result = vsnprintf(space, sizeof(space), format, backup_ap);
+  va_end(backup_ap);
+
+  if ((result >= 0) && (result < sizeof(space))) {
+    // It fit
+    dst->append(space, result);
+    return;
+  }
+
+  // Repeatedly increase buffer size until it fits
+  int length = sizeof(space);
+  while (true) {
+    if (result < 0) {
+      // Older behavior: just try doubling the buffer size
+      length *= 2;
+    } else {
+      // We need exactly "result+1" characters
+      length = result+1;
+    }
+    char* buf = new char[length];
+
+    // Restore the va_list before we use it again
+    va_copy(backup_ap, ap);
+    result = vsnprintf(buf, length, format, backup_ap);
+    va_end(backup_ap);
+
+    if ((result >= 0) && (result < length)) {
+      // It fit
+      dst->append(buf, result);
+      delete[] buf;
+      return;
+    }
+    delete[] buf;
+  }
+}
+
 }   // namespace
 
 namespace mozc {
@@ -462,6 +515,21 @@ void Util::SubString(const string &src,
   return;
 }
 
+bool Util::StartsWith(const string &str, const string &prefix) {
+  if (str.size() < prefix.size()) {
+    return false;
+  }
+  return (0 == memcmp(str.data(), prefix.data(), prefix.size()));
+}
+
+bool Util::EndsWith(const string &str, const string &suffix) {
+  if (str.size() < suffix.size()) {
+    return false;
+  }
+  return (0 == memcmp(str.data() + str.size() - suffix.size(),
+                      suffix.data(), suffix.size()));
+}
+
 void Util::StripUTF8BOM(string *line) {
   const char kUTF8BOM[] = "\xef\xbb\xbf";
   if (line->substr(0, 3) == kUTF8BOM) {
@@ -561,6 +629,40 @@ bool Util::SafeStrToUInt64(const string &str, uint64 *value) {
 #endif
 }
 
+bool Util::SafeStrToDouble(const string &str, double *value) {
+  DCHECK(value);
+
+  const char *s = str.c_str();
+
+  char *endptr;
+  errno = 0;  // errno only gets set on errors
+  // strtod of GCC accepts hexadecimal number like "0x1234", but that of
+  // VisualC++ does not.
+  *value = strtod(s, &endptr);
+  if ((*value ==  numeric_limits<double>::infinity()) ||
+      (*value == -numeric_limits<double>::infinity())) {
+    return false;
+  }
+
+  if (endptr == s) {
+    return false;
+  }
+  while (isspace(*endptr)) {
+    ++endptr;
+  }
+
+  return (*endptr == '\0') && (errno == 0);
+}
+
+string Util::StringPrintf(const char* format, ...) {
+  va_list ap;
+  va_start(ap, format);
+  string result;
+  StringAppendV(&result, format, ap);
+  va_end(ap);
+  return result;
+}
+
 bool Util::ChopReturns(string *line) {
   const string::size_type line_end = line->find_last_not_of("\r\n");
   if (line_end + 1 != line->size()) {
@@ -1068,14 +1170,14 @@ bool Util::IsFullWidthSymbolInHalfWidthKatakana(const string &input) {
     size_t mblen = 0;
     uint16 w = UTF8ToUCS2(begin, end, &mblen);
     switch (w) {
-      case 0x3002:  // FULLSTOP
-      case 0x300C:  // LEFT CORNER BRACKET
-      case 0x300D:  // RIGHT CORNER BRACKET
-      case 0x3001:  // COMMA
-      case 0x30FB:  // MIDDLE DOT
-      case 0x30FC:  // SOUND_MARK
-      case 0x3099:  // VOICE SOUND MARK
-      case 0x309A:  // SEMI VOICE SOUND MARK
+      case 0x3002:  // FULLSTOP "。"
+      case 0x300C:  // LEFT CORNER BRACKET "「"
+      case 0x300D:  // RIGHT CORNER BRACKET "」"
+      case 0x3001:  // COMMA "、"
+      case 0x30FB:  // MIDDLE DOT "・"
+      case 0x30FC:  // SOUND_MARK "ー"
+      case 0x3099:  // VOICE SOUND MARK "゙"
+      case 0x309A:  // SEMI VOICE SOUND MARK "゚"
         begin += mblen;
         break;
       default:
@@ -1093,14 +1195,14 @@ bool Util::IsHalfWidthKatakanaSymbol(const string &input) {
     size_t mblen = 0;
     uint16 w = UTF8ToUCS2(begin, end, &mblen);
     switch (w) {
-      case 0xFF61:  // FULLSTOP
-      case 0xFF62:  // LEFT CORNER BRACKET
-      case 0xFF63:  // RIGHT CORNER BRACKET
-      case 0xFF64:  // COMMA
-      case 0xFF65:  // MIDDLE DOT
-      case 0xFF70:  // SOUND_MARK
-      case 0xFF9E:  // VOICE SOUND MARK
-      case 0xFF9F:  // SEMI VOICE SOUND MARK
+      case 0xFF61:  // FULLSTOP "｡"
+      case 0xFF62:  // LEFT CORNER BRACKET "｢"
+      case 0xFF63:  // RIGHT CORNER BRACKET "｣"
+      case 0xFF64:  // COMMA "､"
+      case 0xFF65:  // MIDDLE DOT "･"
+      case 0xFF70:  // SOUND_MARK "ｰ"
+      case 0xFF9E:  // VOICE SOUND MARK "ﾞ"
+      case 0xFF9F:  // SEMI VOICE SOUND MARK "ﾟ"
         begin += mblen;
         break;
       default:
@@ -1110,6 +1212,37 @@ bool Util::IsHalfWidthKatakanaSymbol(const string &input) {
   return true;
 }
 
+bool Util::IsKanaSymbolContained(const string &input) {
+  const char *begin = input.data();
+  const char *end = begin + input.size();
+  while (begin < end) {
+    size_t mblen = 0;
+    uint16 w = UTF8ToUCS2(begin, end, &mblen);
+    switch (w) {
+      case 0x3002:  // FULLSTOP "。"
+      case 0x300C:  // LEFT CORNER BRACKET "「"
+      case 0x300D:  // RIGHT CORNER BRACKET "」"
+      case 0x3001:  // COMMA "、"
+      case 0x30FB:  // MIDDLE DOT "・"
+      case 0x30FC:  // SOUND_MARK "ー"
+      case 0x3099:  // VOICE SOUND MARK "゙"
+      case 0x309A:  // SEMI VOICE SOUND MARK "゚"
+      case 0xFF61:  // FULLSTOP "｡"
+      case 0xFF62:  // LEFT CORNER BRACKET "｢"
+      case 0xFF63:  // RIGHT CORNER BRACKET "｣"
+      case 0xFF64:  // COMMA "､"
+      case 0xFF65:  // MIDDLE DOT "･"
+      case 0xFF70:  // SOUND_MARK "ｰ"
+      case 0xFF9E:  // VOICE SOUND MARK "ﾞ"
+      case 0xFF9F:  // SEMI VOICE SOUND MARK "ﾟ"
+        return true;
+      default:
+        begin += mblen;
+    }
+  }
+  return false;
+}
+
 bool Util::Unlink(const string &filename) {
 #ifdef OS_WINDOWS
   wstring wide;
@@ -1893,40 +2026,57 @@ Util::ScriptType Util::GetScriptType(const char *begin,
   return Util::GetScriptType(w);
 }
 
-Util::ScriptType Util::GetScriptType(const string &str) {
+namespace {
+Util::ScriptType GetScriptTypeInternal(const string &str,
+                                       bool ignore_symbols) {
   const char *begin = str.data();
   const char *end = str.data() + str.size();
   size_t mblen = 0;
-  ScriptType result = SCRIPT_TYPE_SIZE;
+  Util::ScriptType result = Util::SCRIPT_TYPE_SIZE;
 
   while (begin < end) {
     const uint16 w = Util::UTF8ToUCS2(begin, end, &mblen);
-    ScriptType type = Util::GetScriptType(w);
+    Util::ScriptType type = Util::GetScriptType(w);
     if ((w == 0x30FC || w == 0x30FB ||
          (w >= 0x3099 && w <= 0x309C)) &&
         // PROLONGEDSOUND MARK|MIDLE_DOT|VOICDE_SOUND_MARKS
         // are HIRAGANA as well
-        (result == SCRIPT_TYPE_SIZE ||
-         result == HIRAGANA || result == KATAKANA)) {
+        (result == Util::SCRIPT_TYPE_SIZE ||
+         result == Util::HIRAGANA || result == Util::KATAKANA)) {
       type = result;  // restore the previous state
     }
-    if (type == UNKNOWN_SCRIPT) {
-      return UNKNOWN_SCRIPT;
+    // Ignore symbols
+    // Regard UNKNOWN_SCRIPT as symbols here
+    if (ignore_symbols &&
+        result != Util::UNKNOWN_SCRIPT &&
+        type == Util::UNKNOWN_SCRIPT) {
+      begin += mblen;
+      continue;
     }
     // not first character
-    if (str.data() != begin && result != SCRIPT_TYPE_SIZE && type != result) {
-      return UNKNOWN_SCRIPT;
+    if (str.data() != begin &&
+        result != Util::SCRIPT_TYPE_SIZE && type != result) {
+      return Util::UNKNOWN_SCRIPT;
     }
     result = type;
     begin += mblen;
   }
 
-  if (result == SCRIPT_TYPE_SIZE) {  // everything is "ー"
-    return UNKNOWN_SCRIPT;
+  if (result == Util::SCRIPT_TYPE_SIZE) {  // everything is "ー"
+    return Util::UNKNOWN_SCRIPT;
   }
 
   return result;
 }
+}  // anonymous namespace
+
+Util::ScriptType Util::GetScriptType(const string &str) {
+  return GetScriptTypeInternal(str, false);
+}
+
+Util::ScriptType Util::GetScriptTypeWithoutSymbols(const string &str) {
+  return GetScriptTypeInternal(str, true);
+}
 
 // return true if all script_type in str is "type"
 bool Util::IsScriptType(const string &str, Util::ScriptType type) {
@@ -2124,7 +2274,7 @@ class IsWindowsX64Cache {
     ::ZeroMemory(&system_info, sizeof(system_info));
     proc(&system_info);
 
-    return system_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64;
+    return (system_info.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64);
   }
 
   bool is_x64_;
@@ -2132,7 +2282,33 @@ class IsWindowsX64Cache {
 };
 
 bool Util::IsWindowsX64() {
-  return Singleton<IsWindowsX64Cache>::get()->is_x64();
+  switch (g_is_windows_x64_mode) {
+    case IS_WINDOWS_X64_EMULATE_32BIT_MACHINE:
+      return false;
+    case IS_WINDOWS_X64_EMULATE_64BIT_MACHINE:
+      return true;
+    case IS_WINDOWS_X64_DEFAULT_MODE:
+      return Singleton<IsWindowsX64Cache>::get()->is_x64();
+    default:
+      DCHECK(false) << "Unexpected mode specified.  mode = "
+                    << g_is_windows_x64_mode;
+      return Singleton<IsWindowsX64Cache>::get()->is_x64();
+  }
+}
+
+void Util::SetIsWindowsX64ModeForTest(IsWindowsX64Mode mode) {
+  g_is_windows_x64_mode = mode;
+  switch (g_is_windows_x64_mode) {
+    case IS_WINDOWS_X64_EMULATE_32BIT_MACHINE:
+    case IS_WINDOWS_X64_EMULATE_64BIT_MACHINE:
+    case IS_WINDOWS_X64_DEFAULT_MODE:
+      // Known mode. OK.
+      break;
+    default:
+      DCHECK(false) << "Unexpected mode specified.  mode = "
+                    << g_is_windows_x64_mode;
+      break;
+  }
 }
 
 namespace {
@@ -2343,6 +2519,7 @@ uint64 Util::GetTotalPhysicalMemory() {
   const long number_of_phyisical_pages = sysconf(_SC_PHYS_PAGES);
   if (number_of_phyisical_pages < 0) {
     // likely to be overflowed.
+    LOG(FATAL) << number_of_phyisical_pages << ", " << page_size;
     return 0;
   }
   return static_cast<uint64>(number_of_phyisical_pages) * page_size;
