@@ -74,8 +74,10 @@ class ComposerTest : public testing::Test {
   ComposerTest() {}
 
   virtual void SetUp() {
-    config::ConfigHandler::GetConfig(&default_config_);
     Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+    config::Config config;
+    config::ConfigHandler::GetDefaultConfig(&config);
+    config::ConfigHandler::SetConfig(config);
     table_.reset(new Table);
     composer_.reset(Composer::Create(table_.get()));
     CharacterFormManager::GetCharacterFormManager()->SetDefaultRule();
@@ -84,13 +86,14 @@ class ComposerTest : public testing::Test {
   virtual void TearDown() {
     table_.reset();
     composer_.reset();
-    config::ConfigHandler::SetConfig(default_config_);
+    // just in case, reset config in test_tmpdir
+    config::Config config;
+    config::ConfigHandler::GetDefaultConfig(&config);
+    config::ConfigHandler::SetConfig(config);
   }
 
   scoped_ptr<Composer> composer_;
   scoped_ptr<Table> table_;
-  config::Config default_config_;
-
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ComposerTest);
@@ -108,6 +111,17 @@ TEST_F(ComposerTest, Reset) {
   EXPECT_EQ(transliteration::HALF_ASCII, composer_->GetInputMode());
 }
 
+TEST_F(ComposerTest, ResetInputMode) {
+  composer_->InsertCharacter("mozuku");
+
+  composer_->SetInputMode(transliteration::FULL_KATAKANA);
+  composer_->SetTemporaryInputMode(transliteration::HALF_ASCII);
+  composer_->ResetInputMode();
+
+  EXPECT_FALSE(composer_->Empty());
+  EXPECT_EQ(transliteration::FULL_KATAKANA, composer_->GetInputMode());
+}
+
 TEST_F(ComposerTest, Empty) {
   composer_->InsertCharacter("mozuku");
   EXPECT_FALSE(composer_->Empty());
@@ -149,9 +163,6 @@ TEST_F(ComposerTest, BackSpace) {
   composer_->GetQueryForConversion(&result);
   EXPECT_EQ("ab", result);
 
-  composer_->BackspaceAt(0);
-  EXPECT_EQ(2, composer_->GetLength());
-  EXPECT_EQ(2, composer_->GetCursor());
   result.clear();
   composer_->GetQueryForConversion(&result);
   EXPECT_EQ("ab", result);
@@ -174,7 +185,67 @@ TEST_F(ComposerTest, BackSpace) {
   EXPECT_EQ("ab", result);
 }
 
+
+TEST_F(ComposerTest, InsertCharacterPreeditAt) {
+  // "あ"
+  table_->AddRule("a", "\xe3\x81\x82", "");
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+  // "う"
+  table_->AddRule("u", "\xe3\x81\x86", "");
+  // "く"
+  table_->AddRule("ku", "\xE3\x81\x8F", "");
+
+  composer_->InsertCharacter("au");
+  string output;
+
+  const transliteration::TransliterationType input_mode =
+      composer_->GetInputMode();
+
+  composer_->InsertCharacterPreeditAt(0, "a");
+  EXPECT_EQ(3, composer_->GetLength());
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->GetStringForPreedit(&output);
+  // "aあう"
+  EXPECT_EQ("a\xE3\x81\x82\xE3\x81\x86", output);
+  // Input mode of composer must not be changed
+  EXPECT_EQ(input_mode, composer_->GetInputMode());
+
+  composer_->MoveCursorLeft();
+  // "い"
+  composer_->InsertCharacterPreeditAt(2, "\xE3\x81\x84");
+  EXPECT_EQ(4, composer_->GetLength());
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->GetStringForPreedit(&output);
+  // "aあいう"
+  EXPECT_EQ("a\xE3\x81\x82\xE3\x81\x84\xE3\x81\x86", output);
+  EXPECT_EQ(input_mode, composer_->GetInputMode());
+
+  composer_->MoveCursorLeft();
+  composer_->InsertCharacterPreeditAt(3, "ku");
+  EXPECT_EQ(6, composer_->GetLength());
+  EXPECT_EQ(2, composer_->GetCursor());
+  composer_->GetStringForPreedit(&output);
+  // "aあいkuう"
+  EXPECT_EQ("a\xE3\x81\x82\xE3\x81\x84ku\xE3\x81\x86", output);
+  EXPECT_EQ(input_mode, composer_->GetInputMode());
+
+  composer_->InsertCharacterPreeditAt(0, "1");
+  composer_->GetStringForPreedit(&output);
+  // "1aあいkuう"
+  EXPECT_EQ("1a\xE3\x81\x82\xE3\x81\x84ku\xE3\x81\x86", output);
+  EXPECT_EQ(input_mode, composer_->GetInputMode());
+
+  // Check the actual input mode of composition
+  composer_->InsertCharacter("a");
+  composer_->GetStringForPreedit(&output);
+  // "1aああいkuう"
+  EXPECT_EQ("1a\xE3\x81\x82\xE3\x81\x82\xE3\x81\x84ku\xE3\x81\x86", output);
+}
+
 TEST_F(ComposerTest, OutputMode) {
+  // This behaviour is based on Kotoeri
+
   // "あ"
   table_->AddRule("a", "\xe3\x81\x82", "");
   // "い"
@@ -206,6 +277,41 @@ TEST_F(ComposerTest, OutputMode) {
             output);
 }
 
+TEST_F(ComposerTest, OutputMode_2) {
+  // This behaviour is based on Kotoeri
+
+  // "あ"
+  table_->AddRule("a", "\xe3\x81\x82", "");
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+  // "う"
+  table_->AddRule("u", "\xe3\x81\x86", "");
+
+  composer_->InsertCharacter("a");
+  composer_->InsertCharacter("i");
+  composer_->InsertCharacter("u");
+
+  string output;
+  composer_->GetStringForPreedit(&output);
+  // "あいう"
+  EXPECT_EQ("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x86", output);
+
+  composer_->MoveCursorLeft();
+  composer_->SetOutputMode(transliteration::FULL_ASCII);
+  composer_->GetStringForPreedit(&output);
+  // "ａｉｕ"
+  EXPECT_EQ("\xEF\xBD\x81\xEF\xBD\x89\xEF\xBD\x95", output);
+
+  composer_->InsertCharacter("a");
+  composer_->InsertCharacter("i");
+  composer_->InsertCharacter("u");
+  composer_->GetStringForPreedit(&output);
+  // "ａｉｕあいう"
+  EXPECT_EQ("\xEF\xBD\x81\xEF\xBD\x89\xEF\xBD\x95"
+            "\xE3\x81\x82\xE3\x81\x84\xE3\x81\x86",
+            output);
+}
+
 TEST_F(ComposerTest, GetTransliterations) {
   // "あ"
   table_->AddRule("a", "\xe3\x81\x82", "");
@@ -832,12 +938,12 @@ TEST_F(ComposerTest, ShiftKeyOperation) {
     InsertKey("&", composer_.get());  // "D&"
     InsertKey("D", composer_.get());  // "D&D"
     InsertKey("2", composer_.get());  // "D&D2"
-    InsertKey("a", composer_.get());  // "D&D2あ"
+    InsertKey("a", composer_.get());  // "D&D2a"
 
     string preedit;
     composer_->GetStringForPreedit(&preedit);
-    // "D&D2あ"
-    EXPECT_EQ("\x44\x26\x44\x32\xE3\x81\x82", preedit);
+    // "D&D2a"
+    EXPECT_EQ("\x44\x26\x44\x32\x61", preedit);
   }
 
   {  // Full-witdh alphanumeric
@@ -1291,6 +1397,14 @@ TEST_F(ComposerTest, UpdateInputMode) {
   // "Aいａｉ|"
   composer_->MoveCursorToEnd();
   EXPECT_EQ(transliteration::FULL_KATAKANA, composer_->GetInputMode());
+
+  // "Aいａ|ｉ"
+  composer_->MoveCursorLeft();
+  EXPECT_EQ(transliteration::FULL_ASCII, composer_->GetInputMode());
+
+  // "Aいａｉ|"
+  composer_->MoveCursorToEnd();
+  EXPECT_EQ(transliteration::FULL_KATAKANA, composer_->GetInputMode());
 }
 
 
@@ -1309,6 +1423,66 @@ TEST_F(ComposerTest, TransformCharactersForNumbers) {
   // "−１"
   EXPECT_EQ("\xE2\x88\x92\xEF\xBC\x91", query);
 
+  // "ーー１"
+  query = "\xE3\x83\xBC\xE3\x83\xBC\xEF\xBC\x91";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ー"
+  query = "\xE3\x83\xBC";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーー"
+  query = "\xE3\x83\xBC\xE3\x83\xBC";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーーーーー"
+  query = "\xE3\x83\xBC\xE3\x83\xBC\xE3\x83\xBC\xE3\x83\xBC\xE3\x83\xBC";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ｗ"
+  query = "\xEF\xBD\x97";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーｗ"
+  query = "\xE3\x83\xBC\xEF\xBD\x97";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーーｗ"
+  query = "\xE3\x83\xBC\xE3\x83\xBC\xEF\xBD\x97";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "@" (half-width @)
+  query = "@";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ー@" (half-width @)
+  query = "\xE3\x83\xBC@";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーー@" (half-width @)
+  query = "\xE3\x83\xBC\xE3\x83\xBC@";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "＠" (full-width @)
+  query = "\xEF\xBC\xA0";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ー＠" (full-width @)
+  query = "\xE3\x83\xBC\xEF\xBC\xA0";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "ーー＠" (full-width @)
+  query = "\xE3\x83\xBC\xE3\x83\xBC\xEF\xBC\xA0";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "まじかー１"
+  query = "\xE3\x81\xBE\xE3\x81\x98\xE3\x81\x8B\xE3\x83\xBC\xEF\xBC\x91";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
+  // "まじかーｗ"
+  query = "\xE3\x81\xBE\xE3\x81\x98\xE3\x81\x8B\xE3\x83\xBC\xEF\xBD\x97";
+  EXPECT_FALSE(Composer::TransformCharactersForNumbers(&query));
+
   // "１、０"
   query = "\xEF\xBC\x91\xE3\x80\x81\xEF\xBC\x90";
   EXPECT_TRUE(Composer::TransformCharactersForNumbers(&query));
@@ -1395,10 +1569,22 @@ TEST_F(ComposerTest, TransformCharactersForNumbers) {
           "\xE3\x80\x82\xEF\xBC\x91\xEF\xBC\x94\xEF\xBC\x91\xEF\xBC\x95"
           "\xEF\xBC\x99\xE3\x80\x82";
   EXPECT_TRUE(Composer::TransformCharactersForNumbers(&query));
-  // "およそ、３．１４１５９。"
+  // "およそ、３．１４１５９．"
   EXPECT_EQ("\xE3\x81\x8A\xE3\x82\x88\xE3\x81\x9D\xE3\x80\x81\xEF\xBC\x93"
             "\xEF\xBC\x8E\xEF\xBC\x91\xEF\xBC\x94\xEF\xBC\x91\xEF\xBC\x95"
-            "\xEF\xBC\x99\xE3\x80\x82",
+            "\xEF\xBC\x99\xEF\xBC\x8E",
+            query);
+
+  // "１００、" => "１００，"
+  query = "\xEF\xBC\x91\xEF\xBC\x90\xEF\xBC\x90\xE3\x80\x81";
+  EXPECT_TRUE(Composer::TransformCharactersForNumbers(&query));
+  EXPECT_EQ("\xEF\xBC\x91\xEF\xBC\x90\xEF\xBC\x90\xEF\xBC\x8C",
+            query);
+
+  // "１００。" => "１００．"
+  query = "\xEF\xBC\x91\xEF\xBC\x90\xEF\xBC\x90\xE3\x80\x82";
+  EXPECT_TRUE(Composer::TransformCharactersForNumbers(&query));
+  EXPECT_EQ("\xEF\xBC\x91\xEF\xBC\x90\xEF\xBC\x90\xEF\xBC\x8E",
             query);
 }
 
@@ -1705,5 +1891,441 @@ TEST_F(ComposerTest, Isue2555503) {
   EXPECT_EQ(transliteration::FULL_KATAKANA, composer_->GetInputMode());
 }
 
+TEST_F(ComposerTest, Issue2819580_1) {
+  // This is a unittest against http://b/2819580.
+  // 'y' after 'n' disappears.
+  // "ん"
+  table_->AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table_->AddRule("na", "\xe3\x81\xaa", "");
+  // "や"
+  table_->AddRule("ya", "\xe3\x82\x84", "");
+  // "にゃ"
+  table_->AddRule("nya", "\xe3\x81\xab\xe3\x82\x83", "");
+
+  InsertKey("n", composer_.get());
+  InsertKey("y", composer_.get());
+
+  string result;
+  composer_->GetQueryForConversion(&result);
+  // "んy"
+  EXPECT_EQ("\xe3\x82\x93y", result);
+}
+
+TEST_F(ComposerTest, Issue2819580_2) {
+  // This is a unittest against http://b/2819580.
+  // 'y' after 'n' disappears.
+  // "ぽ"
+  table_->AddRule("po", "\xe3\x81\xbd", "");
+  // "ん"
+  table_->AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table_->AddRule("na", "\xe3\x81\xaa", "");
+  // "や"
+  table_->AddRule("ya", "\xe3\x82\x84", "");
+  // "にゃ"
+  table_->AddRule("nya", "\xe3\x81\xab\xe3\x82\x83", "");
+
+  InsertKey("p", composer_.get());
+  InsertKey("o", composer_.get());
+  InsertKey("n", composer_.get());
+  InsertKey("y", composer_.get());
+
+  string result;
+  composer_->GetQueryForConversion(&result);
+  // "ぽんy"
+  EXPECT_EQ("\xe3\x81\xbd\xe3\x82\x93y", result);
+}
+
+TEST_F(ComposerTest, Issue2819580_3) {
+  // This is a unittest against http://b/2819580.
+  // 'y' after 'n' disappears.
+  // "ん"
+  table_->AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table_->AddRule("na", "\xe3\x81\xaa", "");
+  // "や"
+  table_->AddRule("ya", "\xe3\x82\x84", "");
+  // "にゃ"
+  table_->AddRule("nya", "\xe3\x81\xab\xe3\x82\x83", "");
+
+  InsertKey("z", composer_.get());
+  InsertKey("n", composer_.get());
+  InsertKey("y", composer_.get());
+
+  string result;
+  composer_->GetQueryForConversion(&result);
+  // "zんy"
+  EXPECT_EQ("z\xe3\x82\x93y", result);
+}
+
+TEST_F(ComposerTest, Issue2797991_1) {
+  // This is a unittest against http://b/2797991.
+  // Half-width alphanumeric mode quits after [CAPITAL LETTER]:[CAPITAL LETTER]
+  // e.g. C:\Wi -> C:\Wい
+
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+
+  InsertKey("C", composer_.get());
+  InsertKey(":", composer_.get());
+  InsertKey("\\", composer_.get());
+  InsertKey("W", composer_.get());
+  InsertKey("i", composer_.get());
+
+  string result;
+  composer_->GetStringForPreedit(&result);
+  EXPECT_EQ("C:\\Wi", result);
+}
+
+TEST_F(ComposerTest, Issue2797991_2) {
+  // This is a unittest against http://b/2797991.
+  // Half-width alphanumeric mode quits after [CAPITAL LETTER]:[CAPITAL LETTER]
+  // e.g. C:\Wi -> C:\Wい
+
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+
+  InsertKey("C", composer_.get());
+  InsertKey(":", composer_.get());
+  InsertKey("W", composer_.get());
+  InsertKey("i", composer_.get());
+
+  string result;
+  composer_->GetStringForPreedit(&result);
+  EXPECT_EQ("C:Wi", result);
+}
+
+TEST_F(ComposerTest, Issue2797991_3) {
+  // This is a unittest against http://b/2797991.
+  // Half-width alphanumeric mode quits after [CAPITAL LETTER]:[CAPITAL LETTER]
+  // e.g. C:\Wi -> C:\Wい
+
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+
+  InsertKey("C", composer_.get());
+  InsertKey(":", composer_.get());
+  InsertKey("\\", composer_.get());
+  InsertKey("W", composer_.get());
+  InsertKey("i", composer_.get());
+  InsertKeyWithMode("i", commands::HIRAGANA, composer_.get());
+  string result;
+  composer_->GetStringForPreedit(&result);
+  // "C:\Wiい"
+  EXPECT_EQ("C:\\Wi\xe3\x81\x84", result);
+}
+
+TEST_F(ComposerTest, Issue2797991_4) {
+  // This is a unittest against http://b/2797991.
+  // Half-width alphanumeric mode quits after [CAPITAL LETTER]:[CAPITAL LETTER]
+  // e.g. C:\Wi -> C:\Wい
+
+  // "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+
+  InsertKey("c", composer_.get());
+  InsertKey(":", composer_.get());
+  InsertKey("\\", composer_.get());
+  InsertKey("W", composer_.get());
+  InsertKey("i", composer_.get());
+
+  string result;
+  composer_->GetStringForPreedit(&result);
+  EXPECT_EQ("c:\\Wi", result);
+}
+
+TEST_F(ComposerTest, CaseSensitiveByConfiguration) {
+  {
+    config::Config config;
+    config::ConfigHandler::GetDefaultConfig(&config);
+    config.set_shift_key_mode_switch(mozc::config::Config::OFF);
+    config::ConfigHandler::SetConfig(config);
+    EXPECT_TRUE(mozc::config::ConfigHandler::SetConfig(config));
+    table_->Initialize();
+
+    // i -> "い"
+    table_->AddRule("i", "\xe3\x81\x84", "");
+    // I -> "イ"
+    table_->AddRule("I", "\xe3\x82\xa4", "");
+
+    InsertKey("i", composer_.get());
+    InsertKey("I", composer_.get());
+    InsertKey("i", composer_.get());
+    InsertKey("I", composer_.get());
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "いイいイ"
+    EXPECT_EQ("\xe3\x81\x84\xe3\x82\xa4\xe3\x81\x84\xe3\x82\xa4", result);
+  }
+  composer_->Reset();
+  {
+    config::Config config;
+    config::ConfigHandler::GetDefaultConfig(&config);
+    config.set_shift_key_mode_switch(mozc::config::Config::ASCII_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+    EXPECT_TRUE(mozc::config::ConfigHandler::SetConfig(config));
+    table_->Initialize();
+
+    // i -> "い"
+    table_->AddRule("i", "\xe3\x81\x84", "");
+    // I -> "イ"
+    table_->AddRule("I", "\xe3\x82\xa4", "");
+
+    InsertKey("i", composer_.get());
+    InsertKey("I", composer_.get());
+    InsertKey("i", composer_.get());
+    InsertKey("I", composer_.get());
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "いIiI"
+    EXPECT_EQ("\xe3\x81\x84IiI", result);
+  }
+}
+
+TEST_F(ComposerTest,
+       InputUppercaseInAlphanumericModeWithShiftKeyModeSwitchIsKatakana) {
+  {
+    config::Config config;
+    config::ConfigHandler::GetDefaultConfig(&config);
+    config.set_shift_key_mode_switch(mozc::config::Config::KATAKANA_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+    EXPECT_TRUE(mozc::config::ConfigHandler::SetConfig(config));
+    table_->Initialize();
+
+    // i -> "い"
+    table_->AddRule("i", "\xe3\x81\x84", "");
+    // I -> "イ"
+    table_->AddRule("I", "\xe3\x82\xa4", "");
+
+    {
+      composer_->Reset();
+      composer_->SetInputMode(transliteration::FULL_ASCII);
+      InsertKey("I", composer_.get());
+      string result;
+      composer_->GetStringForPreedit(&result);
+      // "Ｉ"
+      EXPECT_EQ("\xef\xbc\xa9", result);
+    }
+
+    {
+      composer_->Reset();
+      composer_->SetInputMode(transliteration::HALF_ASCII);
+      InsertKey("I", composer_.get());
+      string result;
+      composer_->GetStringForPreedit(&result);
+      EXPECT_EQ("I", result);
+    }
+
+    {
+      composer_->Reset();
+      composer_->SetInputMode(transliteration::FULL_KATAKANA);
+      InsertKey("I", composer_.get());
+      string result;
+      composer_->GetStringForPreedit(&result);
+      // "イ"
+      EXPECT_EQ("\xe3\x82\xa4", result);
+    }
+
+    {
+      composer_->Reset();
+      composer_->SetInputMode(transliteration::HALF_KATAKANA);
+      InsertKey("I", composer_.get());
+      string result;
+      composer_->GetStringForPreedit(&result);
+      // "ｲ"
+      EXPECT_EQ("\xEF\xBD\xB2", result);
+    }
+
+    {
+      composer_->Reset();
+      composer_->SetInputMode(transliteration::HIRAGANA);
+      InsertKey("I", composer_.get());
+      string result;
+      composer_->GetStringForPreedit(&result);
+      // "イ"
+      EXPECT_EQ("\xe3\x82\xa4", result);
+    }
+  }
+}
+
+TEST_F(ComposerTest,
+       DeletingAlphanumericPartShouldQuitToggleAlphanumericMode) {
+  // http://b/2206560
+  // 1. Type "iGoogle" (preedit text turns to be "いGoogle")
+  // 2. Type Back-space 6 times ("い")
+  // 3. Type "i" (should be "いい")
+
+  table_->Initialize();
+
+  // i -> "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+
+  InsertKey("i", composer_.get());
+  InsertKey("G", composer_.get());
+  InsertKey("o", composer_.get());
+  InsertKey("o", composer_.get());
+  InsertKey("g", composer_.get());
+  InsertKey("l", composer_.get());
+  InsertKey("e", composer_.get());
+
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "いGoogle"
+    EXPECT_EQ("\xe3\x81\x84Google", result);
+  }
+
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "い"
+    EXPECT_EQ("\xe3\x81\x84", result);
+  }
+
+  InsertKey("i", composer_.get());
+
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "いい"
+    EXPECT_EQ("\xe3\x81\x84\xe3\x81\x84", result);
+  }
+}
+
+TEST_F(ComposerTest, CursorMoving) {
+  // The expectation of this test is the same as MS-IME's
+
+  table_->Initialize();
+
+  // i -> "い"
+  table_->AddRule("i", "\xe3\x81\x84", "");
+  // gi -> "ぎ"
+  table_->AddRule("gi", "\xe3\x81\x8e", "");
+
+  InsertKey("i", composer_.get());
+  composer_->MoveCursorRight();
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "い"
+    EXPECT_EQ("\xe3\x81\x84", result);
+  }
+
+  composer_->MoveCursorLeft();
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "い"
+    EXPECT_EQ("\xe3\x81\x84", result);
+  }
+
+  InsertKey("G", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "Gい"
+    EXPECT_EQ("G\xe3\x81\x84", result);
+  }
+
+  composer_->MoveCursorRight();
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "Gい"
+    EXPECT_EQ("G\xe3\x81\x84", result);
+  }
+
+  InsertKey("G", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいG"
+    EXPECT_EQ("G\xe3\x81\x84G", result);
+  }
+
+  composer_->MoveCursorLeft();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいいG"
+    EXPECT_EQ("G\xe3\x81\x84\xe3\x81\x84G", result);
+  }
+
+  composer_->MoveCursorRight();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいいGi"
+    EXPECT_EQ("G\xe3\x81\x84\xe3\x81\x84Gi", result);
+  }
+
+  InsertKey("G", composer_.get());
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいいGiGi"
+    EXPECT_EQ("G\xe3\x81\x84\xe3\x81\x84GiGi", result);
+  }
+
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいいGi"
+    EXPECT_EQ("G\xe3\x81\x84\xe3\x81\x84Gi", result);
+  }
+
+  InsertKey("G", composer_.get());
+  InsertKey("G", composer_.get());
+  composer_->MoveCursorRight();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "GいいGiGGi"
+    EXPECT_EQ("G\xe3\x81\x84\xe3\x81\x84GiGGi", result);
+  }
+
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  composer_->Backspace();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "Gい"
+    EXPECT_EQ("G\xe3\x81\x84", result);
+  }
+
+  composer_->Backspace();
+  composer_->MoveCursorLeft();
+  composer_->MoveCursorRight();
+  InsertKey("i", composer_.get());
+  {
+    string result;
+    composer_->GetStringForPreedit(&result);
+    // "Gi"
+    EXPECT_EQ("Gi", result);
+  }
+}
+
 }  // namespace composer
 }  // namespace mozc
